﻿Public Class frmUSB
    ' vendor and product IDs
    Private VendorID As Short
    Private ProductID As Short

    ' read and write buffers
    Private BufferIn() As Byte          ' received data will be stored here - the first byte, BufferIn(0), is "unused" (well it is, but should ALWAYS equal 0)
    Private BufferOut() As Byte         ' transmitted data is stored here - as before, the first item BufferOut(0) must always equal 0
    ' number of elements per buffer
    Private BufferInSize As Byte
    Private BufferOutSize As Byte

    ' ****************************************************************
    ' update settings and set default form values
    ' in preperation to connect to a device
    '*****************************************************************
    Private Sub UpdateUSB_IDs()
        Try
            ' set device information
            VendorID = CShort(txtVendorID.Text)
            ProductID = CShort(txtProductID.Text)
            ReDim BufferIn(CByte(txtBufferInSize.Text))
            ReDim BufferOut(CByte(txtBufferOutSize.Text))
            BufferInSize = CByte(txtBufferInSize.Text)
            BufferOutSize = CByte(txtBufferOutSize.Text)

            ' redraw/set output report list box and data
            lstReport.Items.Clear()
            Dim i As Integer
            For i = 0 To BufferOutSize - 1
                lstReport.Items.Add(Trim(Str(i)))
            Next

            ' reset form fields
            txtInfo.Clear()
            txtConnects.Text = "0"
            txtRX.Text = "0"
            txtTX.Text = "0"
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    ' ****************************************************************
    ' when the form loads, connect to the HID controller - pass
    ' the form window handle so that you can receive notification
    ' events...
    '*****************************************************************
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        If (System.Deployment.Application.ApplicationDeployment.IsNetworkDeployed) Then
            Me.Text = "iHID by Graham Vr " & My.Application.Deployment.CurrentVersion.ToString
        End If
        Me.FormBorderStyle = FormBorderStyle.FixedSingle

        ' grab default settings from form for desired HID
        UpdateUSB_IDs()
        ' attempt to connect to the device
        ConnectToHID(Me.Handle)
    End Sub

    '*****************************************************************
    ' disconnect from the HID controller...
    '*****************************************************************
    Private Sub Form1_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        DisconnectFromHID()
    End Sub

    '*****************************************************************
    ' a HID device has been plugged in...
    '*****************************************************************
    Public Sub OnPlugged(ByVal pHandle As Integer)
        If hidGetVendorID(pHandle) = VendorID And hidGetProductID(pHandle) = ProductID Then
            ' device plugged in.. 
            txtInfo.AppendText("Device plugged in @ " & DateTime.Now & vbCrLf)
            txtConnects.Text = Trim(Str(CLng(txtConnects.Text) + 1))
            picPluggedIn.Visible = True
            picUnplugged.Visible = False
            cmdSendReport.Enabled = True
            cmdPause.Enabled = True
            cmdResume.Enabled = False
        End If
    End Sub

    '*****************************************************************
    ' a HID device has been unplugged...
    '*****************************************************************
    Public Sub OnUnplugged(ByVal pHandle As Integer)
        If hidGetVendorID(pHandle) = VendorID And hidGetProductID(pHandle) = ProductID Then
            ' device unplugged..
            hidSetReadNotify(hidGetHandle(VendorID, ProductID), False)
            txtInfo.AppendText("Device unplugged @ " & DateTime.Now & vbCrLf)
            picPluggedIn.Visible = False
            picUnplugged.Visible = True
            cmdSendReport.Enabled = False
            cmdPause.Enabled = False
            cmdResume.Enabled = False
        End If
    End Sub

    '*****************************************************************
    ' controller changed notification - called
    ' after ALL HID devices are plugged or unplugged
    '*****************************************************************
    Public Sub OnChanged()
        ' get the handle of the device we are interested in, then set
        ' its read notify flag to true - this ensures you get a read
        ' notification message when there is some data to read...
        Dim pHandle As Integer
        pHandle = hidGetHandle(VendorID, ProductID)
        hidSetReadNotify(hidGetHandle(VendorID, ProductID), True)
    End Sub

    '*****************************************************************
    ' on read event...
    '*****************************************************************
    Public Sub OnRead(ByVal pHandle As Integer)
        ' read the data (don't forget, pass the whole array)...
        ' first byte is the report ID, e.g. BufferIn(0)
        ' the other bytes are the data from the microcontroller...
        If hidRead(pHandle, BufferIn(0)) Then
            ' report recieved, parse the buffer..
            Dim i As Integer
            Dim tmpByte As Byte

            ' parse the buffer and appened info textbox
            txtInfo.AppendText("Report Recieved @ " & DateTime.Now & vbCrLf)
            ' physical data starts from BufferIn(1), remember that BufferIn(0) always equals 0.
            For i = 1 To BufferInSize
                tmpByte = BufferIn(i)
                txtInfo.AppendText("Buffer " & Trim(Str(i - 1)) & " = " & Trim(Str(tmpByte)) & vbCrLf)
            Next

            ' clock the number of reports received
            txtRX.Text = Trim(Str(CLng(txtRX.Text) + 1))
        End If
    End Sub

    '*****************************************************************
    ' "Update" button clicked...
    '*****************************************************************
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdUpdate.Click
        ' disconnect from current device
        DisconnectFromHID()
        ' redefine parameters
        UpdateUSB_IDs()
        ' attempt to connect to new device
        ConnectToHID(Me.Handle)
    End Sub

    '*****************************************************************
    ' "Output Report" list clicked...
    '*****************************************************************
    Private Sub lstReport_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles lstReport.Click
        Dim sIndex As Integer
        Dim tmpResponse As String

        sIndex = lstReport.SelectedIndex
        ' ensure list index is valid
        If sIndex = -1 Then Exit Sub

        Try
            ' ask user for new output report data
            tmpResponse = InputBox("Enter byte data for BufferOut(" & Trim(Str(sIndex)) & ")", "Edit Output Report Buffer", Trim(Str(lstReport.Items(sIndex))))
            If tmpResponse <> "" Then
                ' convert the response to byte format and save to output report list
                lstReport.Items(sIndex) = CByte(tmpResponse)
            End If
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    '*****************************************************************
    ' "Send" clicked...
    '*****************************************************************
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSendReport.Click
        Dim i As Integer

        ' Action any pending events that may have triggered
        '  in this case, the number of transmitted packets will be refreshed 
        '  as it will not do so if multiple packets have been sent in a row.
        Application.DoEvents()

        ' Load the output buffer from the lstbox items
        BufferOut(0) = 0
        For i = 1 To BufferOutSize
            BufferOut(i) = CByte(lstReport.Items(i - 1))
        Next

        ' Get the handle
        Dim pHandle As Integer
        pHandle = hidGetHandle(VendorID, ProductID)

        Try
            ' envoke the following line to send the entire report
            hidWriteEx(VendorID, ProductID, BufferOut(0))

            ' update info textbox with time, date and report contents
            txtInfo.AppendText("Report Sent @ " & DateTime.Now & vbCrLf)
            For i = 1 To BufferOutSize
                txtInfo.AppendText("Buffer " & Trim(Str(i - 1)) & " = " & Trim(Str(BufferOut(i))) & vbCrLf)
            Next
            txtTX.Text = Trim(Str(CLng(txtTX.Text) + 1))
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Sub

    '*****************************************************************
    ' "Exit" clicked...
    '*****************************************************************
    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdExit.Click
        ' disconnect from the device
        DisconnectFromHID()
        ' close the program
        End
    End Sub

    '*****************************************************************
    ' "Pause" clicked...
    '*****************************************************************
    Private Sub cmdStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPause.Click
        ' enable the resume button
        cmdResume.Enabled = True
        cmdPause.Enabled = False

        Try
            ' disable notify events for connected device. Note, device remains connected.
            ' all unhandled events will trigger when enabled again.
            hidSetReadNotify(hidGetHandle(VendorID, ProductID), False)
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    '*****************************************************************
    ' "Resume" clicked...
    '*****************************************************************
    Private Sub cmdStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdResume.Click
        ' enable the pause button
        cmdResume.Enabled = False
        cmdPause.Enabled = True

        Try
            ' enable notify events for the connected device. Note, all pending reports/events
            '  will trigger after the enable.
            hidSetReadNotify(hidGetHandle(VendorID, ProductID), True)
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    '*****************************************************************
    ' other misc subs
    '*****************************************************************
    Private Sub LinkLabel2_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabel2.LinkClicked        
        Process.Start("http://digital-diy.com/forum/viewtopic.php?f=39")
    End Sub

    Private Sub LinkLabel1_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabel1.LinkClicked
        frmUSB_Schematic.Show()
    End Sub

    Private Sub LinkLabel3_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabel3.LinkClicked
        Process.Start("http://digital-diy.com/forum/viewtopic.php?f=39&t=288#p1365")
    End Sub

End Class
